% JBeig.m
% This Matlab script calculates the eigenvalues and eigenvectors
% from the lienarized equations of motion for the bicycle.
% The degrees of freedom are q=[lean angle; steer angle],
% lean to the right and steer to the left is positive
% when driving forward.
% The linearized equations of motion read:
%
%       M0*qdd+(C1.*v)*qd+(K0+K2.*v^2)*q=0
%
% where v is the forward speed of the bike.
% 
% The massmatrix M0, 
% the velocity sensetivity matrix C1 
% the stifness matrices K0 and K2
% must me set by JBmck or read.
%
% The forward speed v is in the range 0 to 10 divided in 101 points.
%
% The eigenvalues are stored in lamb(101,4)
% The eigenvectors are stored as a list of 4*4=16 numbers in Lamb(101,4*4)

% Nov-18-2003 Schwab, changed the eigenvalue problem from A*V=V*D to  A*V = B*V*D
% Created by Arend L. Schwab and Jim Papadopoulos, 18-Jun-2003
% Delft University of Technology & Cornell University.
% Dec-16-2004 Schwab, adapted for wb1443 class

% read the matrices
load BM1M0C1K0K2

max_vel = 10;
npnts = 101;

v=linspace(0, ... %minumum velocity
           max_vel, ... %maximum velocity
           npnts ... %distretization
           );

n=length(v);
% allocate space for the eigenvalues and the eigenvectore
lamb=zeros(n,4);
Lamb=zeros(n,4*4);
% do the eigenvaue problem for all speeds
for i=1:n
  M=M0;
  C=C1.*v(i);
  K=K0+K2.*v(i)^2;
% Eigenvalue problem A*V = B*V*D
  B=[M zeros(2); zeros(2) eye(2)];
  A=[-C -K; eye(2) zeros(2)];
  [V,D]=eig(A,B);
  li=diag(D);
  Li=V;
% sort the lambdas eand the Lambdas
  [ls,ind]=sort(real(li));
  ls=li(ind);
  Ls=V(:,ind);
  lamb(i,:)=ls.';
  Lamb(i,:)=Ls(:).';
end

% figure
% plot(real(lamb),imag(lamb))
% xlabel('Re(\lambda)')
% ylabel('Im(\lambda)')
% grid
% 
% figure
% plot3(real(lamb),imag(lamb),v,'.')
% xlabel('Re(\lambda)')
% ylabel('Im(\lambda)')
% zlabel('v')
% grid
% 
% figure
% plot(v,real(lamb))
% ylabel('Re(\lambda)')
% xlabel('v')
% hold on
% %%plot([v1 v1],[-10 10],'-k')
% grid


% JBplot4


%save BM1Lamb101  v lamb Lamb